import os

import nd2
import numpy as np
import ome_types
from tifffile import tifffile

from nellie import logger


class FileInfo:
    def __init__(self, filepath, output_dir=None):
        self.filepath = filepath
        self.metadata = None
        self.metadata_type = None
        self.axes = None
        self.shape = None
        self.dim_res = None

        self.input_dir = os.path.dirname(filepath)
        self.basename = os.path.basename(filepath)
        self.filename_no_ext = os.path.splitext(self.basename)[0]
        self.extension = os.path.splitext(filepath)[1]
        self.output_dir = output_dir or os.path.join(self.input_dir, 'nellie_output')
        if not os.path.exists(self.output_dir):
            os.makedirs(self.output_dir)

        self.nellie_necessities_dir = os.path.join(self.output_dir, 'nellie_necessities')
        if not os.path.exists(self.nellie_necessities_dir):
            os.makedirs(self.nellie_necessities_dir)

        self.ome_output_path = None
        self.good_dims = False
        self.good_axes = False

        self.ch = 0
        self.t_start = 0
        self.t_end = None
        self.dtype = None

    def _find_tif_metadata(self):
        with tifffile.TiffFile(self.filepath) as tif:
            if tif.is_ome or tif.ome_metadata is not None:
                ome_xml = tifffile.tiffcomment(self.filepath)
                metadata = ome_types.from_xml(ome_xml)
                metadata_type = 'ome'
            elif tif.is_imagej:
                metadata = tif.imagej_metadata
                metadata_type = 'imagej'
                if 'physicalsizex' not in metadata:
                    metadata_type = 'imagej_tif_tags'
                    metadata = [metadata, tif.pages[0].tags._dict]
            else:
                metadata = tif.pages[0].tags._dict
                metadata_type = None

            self.metadata = metadata
            self.metadata_type = metadata_type
            self.axes = tif.series[0].axes
            self.shape = tif.series[0].shape

        return metadata, metadata_type

    def _find_nd2_metadata(self):
        with nd2.ND2File(self.filepath) as nd2_file:
            metadata = nd2_file.metadata.channels[0]
            metadata.recorded_data = nd2_file.events(orient='list')
            self.metadata = metadata
            self.metadata_type = 'nd2'
            self.axes = ''.join(nd2_file.sizes.keys())
            self.shape = tuple(nd2_file.sizes.values())

    def find_metadata(self):
        if self.filepath.endswith('.tiff') or self.filepath.endswith('.tif'):
            self._find_tif_metadata()
        elif self.filepath.endswith('.nd2'):
            self._find_nd2_metadata()
        else:
            raise ValueError('File type not supported')

    def _get_imagej_metadata(self, metadata):
        self.dim_res['X'] = metadata['physicalsizex'] if 'physicalsizex' in metadata else None
        self.dim_res['Y'] = metadata['physicalsizey'] if 'physicalsizey' in metadata else None
        self.dim_res['Z'] = metadata['spacing'] if 'spacing' in metadata else None
        self.dim_res['T'] = metadata['finterval'] if 'finterval' in metadata else None

    def _get_ome_metadata(self, metadata):
        self.dim_res['X'] = metadata.images[0].pixels.physical_size_x
        self.dim_res['Y'] = metadata.images[0].pixels.physical_size_y
        self.dim_res['Z'] = metadata.images[0].pixels.physical_size_z
        self.dim_res['T'] = metadata.images[0].pixels.time_increment

    def _get_tif_tags_metadata(self, metadata):
        tag_names = {tag_value.name: tag_code for tag_code, tag_value in metadata.items()}

        if 'XResolution' in tag_names:
            self.dim_res['X'] = metadata[tag_names['XResolution']].value[1] \
                                  / metadata[tag_names['XResolution']].value[0]
        if 'YResolution' in tag_names:
            self.dim_res['Y'] = metadata[tag_names['YResolution']].value[1] \
                                  / metadata[tag_names['YResolution']].value[0]
        if 'ResolutionUnit' in tag_names:
            if metadata[tag_names['ResolutionUnit']].value == tifffile.RESUNIT.CENTIMETER:
                self.dim_res['X'] *= 1E-4 * 1E6
                self.dim_res['Y'] *= 1E-4 * 1E6
        if 'Z' in self.axes:
            if 'ZResolution' in tag_names:
                self.dim_res['Z'] = 1 / metadata[tag_names['ZResolution']].value[0]
        if 'T' in self.axes:
            if 'FrameRate' in tag_names:
                self.dim_res['T'] = 1 / metadata[tag_names['FrameRate']].value[0]

    def _get_nd2_metadata(self, metadata):
        if 'Time [s]' in metadata.recorded_data:
            timestamps = metadata.recorded_data['Time [s]']
            self.dim_res['T'] = timestamps[-1] / len(timestamps)
        self.dim_res['X'] = metadata.volume.axesCalibration[0]
        self.dim_res['Y'] = metadata.volume.axesCalibration[1]
        self.dim_res['Z'] = metadata.volume.axesCalibration[2]

    def load_metadata(self):
        self.dim_res = {'X': None, 'Y': None, 'Z': None, 'T': None}
        if self.metadata_type == 'ome':
            self._get_ome_metadata(self.metadata)
        elif self.metadata_type == 'imagej':
            self._get_imagej_metadata(self.metadata)
        elif self.metadata_type == 'imagej_tif_tags':
            self._get_imagej_metadata(self.metadata[0])
            self._get_tif_tags_metadata(self.metadata[1])
        elif self.metadata_type == 'nd2':
            self._get_nd2_metadata(self.metadata)
        elif self.metadata_type is None:
            self._get_tif_tags_metadata(self.metadata)
        self._validate()

    def _check_axes(self):
        if len(self.shape) != len(self.axes):
            self.change_axes('Q' * len(self.shape))
        for axis in self.axes:
            if axis not in ['T', 'Z', 'Y', 'X', 'C']:
                self.good_axes = False
                return
        # if any duplicates, not good
        if len(set(self.axes)) != len(self.axes):
            self.good_axes = False
            return
        # if X or Y are not there, not good
        if 'X' not in self.axes or 'Y' not in self.axes:
            self.good_axes = False
            return
        self.good_axes = True

    def _check_dim_res(self):
        check_dims = ['X', 'Y', 'Z', 'T']
        for dim in check_dims:
            if dim in self.axes and self.dim_res[dim] is None:
                self.good_dims = False
                return
        self.good_dims = True

    def change_axes(self, new_axes):
        # if len(new_axes) != len(self.shape):
        self.good_axes = False
            # return
            # raise ValueError('New axes must have the same length as the shape of the data')
        self.axes = new_axes
        self._validate()

    def change_dim_res(self, dim, new_size):
        if dim not in self.dim_res:
            return
            # raise ValueError('Invalid dimension')
        self.dim_res[dim] = new_size
        self._validate()

    def change_selected_channel(self, ch):
        if not self.good_dims or not self.good_axes:
            raise ValueError('Must have both valid axes and dimensions to change channel')
        if 'C' not in self.axes:
            raise KeyError('No channel dimension to change')
        if ch < 0 or ch >= self.shape[self.axes.index('C')]:
            raise IndexError('Invalid channel index')
        self.ch = ch
        self._get_output_path()

    def select_temporal_range(self, start=0, end=None):
        if not self.good_dims or not self.good_axes:
            return
            # raise ValueError('Must have both valid axes and dimensions to select temporal range')
        if 'T' not in self.axes:
            return
            # raise KeyError('No time dimension to select')
        self.t_start = start
        self.t_end = end
        if self.t_end is None:
            self.t_end = self.shape[self.axes.index('T')] - 1
        self._get_output_path()

    def _validate(self):
        self._check_axes()
        self._check_dim_res()
        self.select_temporal_range()
        self._get_output_path()

    def read_file(self):
        if self.extension == '.nd2':
            data = nd2.imread(self.filepath)
        elif self.extension == '.tif' or self.extension == '.tiff':
            try:
                data = tifffile.memmap(self.filepath)
            except:
                data = tifffile.imread(self.filepath)
        else:
            logger.error(f'Filetype {self.extension} not supported. Please convert to .nd2 or .tif.')
            raise ValueError
        self.dtype = data.dtype
        return data

    def _get_output_path(self):
        t_text = f'-t{self.t_start}_to_{self.t_end}' if 'T' in self.axes else ''
        dim_texts = []
        for axis in self.axes:
            if axis not in self.dim_res:
                continue
            dim_res = self.dim_res[axis]
            # round to 4 decimal places
            if dim_res is None:
                dim_res = 'None'
            else:
                dim_res = str(round(dim_res, 4))
            # convert '.' to 'p'
            dim_res = dim_res.replace('.', 'p')
            dim_texts.append(f'{axis}{dim_res}')
        dim_text = f"-{'_'.join(dim_texts)}"
        output_name = f'{self.filename_no_ext}-{self.axes}{dim_text}-ch{self.ch}{t_text}'
        self.user_output_path_no_ext = os.path.join(self.output_dir, output_name)
        self.nellie_necessities_output_path_no_ext = os.path.join(self.nellie_necessities_dir, output_name)
        self.ome_output_path = self.nellie_necessities_output_path_no_ext + '.ome.tif'

    def save_ome_tiff(self):
        if not self.good_axes or not self.good_dims:
            raise ValueError('Cannot save file with invalid axes or dimensions')

        axes = self.axes
        data = self.read_file()
        if 'T' not in self.axes:
            data = data[np.newaxis, ...]
            axes = 'T' + self.axes
        else:
            t_index = self.axes.index('T')
            selected_range = range(self.t_start, self.t_end + 1)
            data = np.take(data, selected_range, axis=t_index)
            # if len(selected_range) == 1:
            #     data = np.expand_dims(data, axis=t_index)
        if 'C' in axes:
            data = np.take(data, self.ch, axis=axes.index('C'))
            axes = axes.replace('C', '')

        # ensure 'T' is the 0th dimension
        if 'T' in axes:
            t_index = axes.index('T')
            data = np.moveaxis(data, t_index, 0)
            axes = 'T' + axes.replace('T', '')

        tifffile.imwrite(
            self.ome_output_path, data, bigtiff=True, metadata={"axes": axes}
        )

        ome_xml = tifffile.tiffcomment(self.ome_output_path)
        ome = ome_types.from_xml(ome_xml)
        ome.images[0].pixels.physical_size_x = self.dim_res['X']
        ome.images[0].pixels.physical_size_y = self.dim_res['Y']
        ome.images[0].pixels.physical_size_z = self.dim_res['Z']
        ome.images[0].pixels.time_increment = self.dim_res['T']
        dtype_name = data.dtype.name
        if data.dtype.name == 'float64':
            dtype_name = 'double'
        if data.dtype.name == 'float32':
            dtype_name = 'float'
        ome.images[0].pixels.type = dtype_name
        ome_xml = ome.to_xml()
        tifffile.tiffcomment(self.ome_output_path, ome_xml)


class ImInfo:
    def __init__(self, file_info: FileInfo):
        self.file_info = file_info
        self.im_path = file_info.ome_output_path
        if not os.path.exists(self.im_path):
            file_info.save_ome_tiff()
        self.im = tifffile.memmap(self.im_path)

        self.screenshot_dir = os.path.join(self.file_info.output_dir, 'screenshots')
        self.graph_dir = os.path.join(self.file_info.output_dir, 'graphs')

        self.dim_res = {'X': None, 'Y': None, 'Z': None, 'T': None}
        self.axes = None
        self.new_axes = None
        self.shape = None
        self.ome_metadata = None
        self._get_ome_metadata()

        self.no_z = True
        self.no_t = True
        self._check_axes_exist()

        self.pipeline_paths = {}
        self._create_output_paths()

    def _check_axes_exist(self):
        if 'Z' in self.axes and self.shape[self.new_axes.index('Z')] > 1:
            self.no_z = False
        if 'T' in self.axes and self.shape[self.new_axes.index('T')] > 1:
            self.no_t = False

    def create_output_path(self, pipeline_path: str, ext: str = '.ome.tif', for_nellie=True):
        if for_nellie:
            output_path = f'{self.file_info.nellie_necessities_output_path_no_ext}-{pipeline_path}{ext}'
        else:
            output_path = f'{self.file_info.user_output_path_no_ext}-{pipeline_path}{ext}'
        self.pipeline_paths[pipeline_path] = output_path
        return self.pipeline_paths[pipeline_path]

    def _create_output_paths(self):
        self.create_output_path('im_preprocessed')
        self.create_output_path('im_instance_label')
        self.create_output_path('im_skel')
        self.create_output_path('im_skel_relabelled')
        self.create_output_path('im_pixel_class')
        self.create_output_path('im_marker')
        self.create_output_path('im_distance')
        self.create_output_path('im_border')
        self.create_output_path('flow_vector_array', ext='.npy')
        self.create_output_path('voxel_matches', ext='.npy')
        self.create_output_path('im_branch_label_reassigned')
        self.create_output_path('im_obj_label_reassigned')
        self.create_output_path('features_voxels', ext='.csv', for_nellie=False)
        self.create_output_path('features_nodes', ext='.csv', for_nellie=False)
        self.create_output_path('features_branches', ext='.csv', for_nellie=False)
        self.create_output_path('features_organelles', ext='.csv', for_nellie=False)
        self.create_output_path('features_image', ext='.csv', for_nellie=False)
        self.create_output_path('adjacency_maps', ext='.pkl')

    def remove_intermediates(self):
        all_pipeline_paths = [self.pipeline_paths[pipeline_path] for pipeline_path in self.pipeline_paths]
        for pipeline_path in all_pipeline_paths + [self.im_path]:
            if 'csv' in pipeline_path:
                continue
            elif os.path.exists(pipeline_path):
                os.remove(pipeline_path)

    def _get_ome_metadata(self, ):
        with tifffile.TiffFile(self.im_path) as tif:
            self.axes = tif.series[0].axes
            self.new_axes = self.axes
        if 'T' not in self.axes:
            self.im = self.im[np.newaxis, ...]
            self.new_axes = 'T' + self.axes
        self.shape = self.im.shape
        self.ome_metadata = ome_types.from_xml(tifffile.tiffcomment(self.im_path))
        self.dim_res['X'] = self.ome_metadata.images[0].pixels.physical_size_x
        self.dim_res['Y'] = self.ome_metadata.images[0].pixels.physical_size_y
        self.dim_res['Z'] = self.ome_metadata.images[0].pixels.physical_size_z
        self.dim_res['T'] = self.ome_metadata.images[0].pixels.time_increment

    def get_memmap(self, file_path, read_mode='r+'):
        memmap = tifffile.memmap(file_path, mode=read_mode)
        if 'T' not in self.axes:
            memmap = memmap[np.newaxis, ...]
        return memmap

    def allocate_memory(self, output_path, dtype='float', data=None, description='No description.',
                        return_memmap=False, read_mode='r+'):
        axes = self.axes
        if 'T' not in self.axes:
            axes = 'T' + axes
            if data is not None:
                data = data[np.newaxis, ...]
        if data is None:
            tifffile.imwrite(
                output_path, shape=self.shape, dtype=dtype, bigtiff=True, metadata={"axes": axes}
            )
        else:
            dtype = dtype or data.dtype.name
            tifffile.imwrite(
                output_path, data, bigtiff=True, metadata={"axes": axes}
            )
        ome = self.ome_metadata
        ome.images[0].description = description
        ome.images[0].pixels.type = dtype
        ome_xml = ome.to_xml()
        tifffile.tiffcomment(output_path, ome_xml)
        if return_memmap:
            return self.get_memmap(output_path, read_mode=read_mode)


if __name__ == "__main__":
    test_dir = '/Users/austin/test_files/nellie_all_tests'
    all_paths = os.listdir(test_dir)
    all_paths = [os.path.join(test_dir, path) for path in all_paths if path.endswith('.tiff') or path.endswith('.tif') or path.endswith('.nd2')]
    # for filepath in all_paths:
    #     file_info = FileInfo(filepath)
    #     file_info.find_metadata()
    #     file_info.load_metadata()
    #     print(file_info.metadata_type)
    #     print(file_info.axes)
    #     print(file_info.shape)
    #     print(file_info.dim_res)
    #     print('\n\n')

    test_file = all_paths[1]
    file_info = FileInfo(test_file)
    file_info.find_metadata()
    file_info.load_metadata()
    print(f'{file_info.metadata_type=}')
    print(f'{file_info.axes=}')
    print(f'{file_info.shape=}')
    print(f'{file_info.dim_res=}')
    print(f'{file_info.good_axes=}')
    print(f'{file_info.good_dims=}')
    print('\n')

    file_info.change_axes('TZYX')
    print('Axes changed')
    print(f'{file_info.axes=}')
    print(f'{file_info.dim_res=}')
    print(f'{file_info.good_axes=}')
    print(f'{file_info.good_dims=}')
    print('\n')

    file_info.change_dim_res('T', 0.5)
    file_info.change_dim_res('Z', 0.2)

    print('Dimension resolutions changed')
    print(f'{file_info.axes=}')
    print(f'{file_info.dim_res=}')
    print(f'{file_info.good_axes=}')
    print(f'{file_info.good_dims=}')
    print('\n')

    # print(f'{file_info.ch=}')
    # file_info.change_selected_channel(3)
    # print('Channel changed')
    # print(f'{file_info.ch=}')

    print(f'{file_info.t_start=}')
    print(f'{file_info.t_end=}')
    file_info.select_temporal_range(1, 3)
    print('Temporal range selected')
    print(f'{file_info.t_start=}')
    print(f'{file_info.t_end=}')

    # file_info.save_ome_tiff()
    im_info = ImInfo(file_info)



from nellie.feature_extraction.hierarchical import Hierarchy
from nellie.im_info.verifier import FileInfo, ImInfo
from nellie.segmentation.filtering import Filter
from nellie.segmentation.labelling import Label
from nellie.segmentation.mocap_marking import Markers
from nellie.segmentation.networking import Network
from nellie.tracking.hu_tracking import HuMomentTracking
from nellie.tracking.voxel_reassignment import VoxelReassigner


def run(file_info, remove_edges=False, otsu_thresh_intensity=False, threshold=None):
    im_info = ImInfo(file_info)
    preprocessing = Filter(im_info, remove_edges=remove_edges)
    preprocessing.run()

    segmenting = Label(im_info, otsu_thresh_intensity=otsu_thresh_intensity, threshold=threshold)
    segmenting.run()

    networking = Network(im_info)
    networking.run()

    mocap_marking = Markers(im_info)
    mocap_marking.run()

    hu_tracking = HuMomentTracking(im_info)
    hu_tracking.run()

    vox_reassign = VoxelReassigner(im_info)
    vox_reassign.run()

    hierarchy = Hierarchy(im_info)
    hierarchy.run()

    return im_info


if __name__ == "__main__":
    # # Single file run
    # im_path = r"/Users/austin/test_files/nellie_all_tests/ND Stimulation Parallel 12.nd2"
    # im_info = run(im_path, remove_edges=False, num_t=5)
    # im_info = run(im_path, remove_edges=False, ch=1, dim_sizes={'T': 1, 'Z': 0.1, 'Y': 0.1, 'X': 0.1}, otsu_thresh_intensity=True)

    # Directory batch run
    # import os
    # top_dirs = [
    #     r"C:\Users\austin\GitHub\nellie-supplemental\comparisons\simulations\multi_grid\outputs",
    #     r"C:\Users\austin\GitHub\nellie-supplemental\comparisons\simulations\separation\outputs",
    #     r"C:\Users\austin\GitHub\nellie-supplemental\comparisons\simulations\px_sizes\outputs",
    #     ]
    # ch = 0
    # num_t = 1
    # # get all non-folder files
    # for top_dir in top_dirs:
    #     all_files = os.listdir(top_dir)
    #     all_files = [os.path.join(top_dir, file) for file in all_files if not os.path.isdir(os.path.join(top_dir, file))]
    #     all_files = [file for file in all_files if file.endswith('.tif')]
    #     for file_num, tif_file in enumerate(all_files):
    #         # for ch in range(1):
    #         print(f'Processing file {file_num + 1} of {len(all_files)}, channel {ch + 1} of 1')
    #         im_info = ImInfo(tif_file, ch=ch)
    #         if os.path.exists(im_info.pipeline_paths['im_skel_relabelled']):
    #             print(f'Already exists, skipping.')
    #             continue
    #         im_info = run(tif_file, remove_edges=False, ch=ch, num_t=num_t)

    test_file = '/Users/austin/test_files/nellie_all_tests/yeast_3d_mitochondria.ome.tif'
    # test_file = all_paths[1]
    file_info = FileInfo(test_file)
    file_info.find_metadata()
    file_info.load_metadata()
    # print(f'{file_info.metadata_type=}')
    # print(f'{file_info.axes=}')
    # print(f'{file_info.shape=}')
    # print(f'{file_info.dim_res=}')
    # print(f'{file_info.good_axes=}')
    # print(f'{file_info.good_dims=}')
    # print('\n')

    # file_info.change_axes('TZYX')
    # print('Axes changed')
    # print(f'{file_info.axes=}')
    # print(f'{file_info.dim_res=}')
    # print(f'{file_info.good_axes=}')
    # print(f'{file_info.good_dims=}')
    # print('\n')
    #
    # file_info.change_dim_res('T', 1)
    # file_info.change_dim_res('Z', 0.5)
    # file_info.change_dim_res('Y', 0.2)
    # file_info.change_dim_res('X', 0.2)
    #
    # print('Dimension resolutions changed')
    # print(f'{file_info.axes=}')
    # print(f'{file_info.dim_res=}')
    # print(f'{file_info.good_axes=}')
    # print(f'{file_info.good_dims=}')
    # print('\n')
    #
    # # print(f'{file_info.ch=}')
    # # file_info.change_selected_channel(3)
    # # print('Channel changed')
    # # print(f'{file_info.ch=}')
    #
    # print(f'{file_info.t_start=}')
    # print(f'{file_info.t_end=}')
    # file_info.select_temporal_range(1, 3)
    # print('Temporal range selected')
    # print(f'{file_info.t_start=}')
    # print(f'{file_info.t_end=}')
    #
    # # file_info.save_ome_tiff()
    # # im_info = ImInfo(file_info)
    run(file_info)



import os
import subprocess
import time

from napari.utils.notifications import show_info
from qtpy.QtWidgets import QWidget, QPushButton, QVBoxLayout, QGroupBox, QLabel, QHBoxLayout
from qtpy.QtGui import QFont, QIcon
from qtpy.QtCore import Qt, QTimer

from nellie.feature_extraction.hierarchical import Hierarchy
from nellie.segmentation.filtering import Filter
from nellie.segmentation.labelling import Label
from nellie.segmentation.mocap_marking import Markers
from nellie.segmentation.networking import Network
from nellie.tracking.hu_tracking import HuMomentTracking
from nellie.tracking.voxel_reassignment import VoxelReassigner
from napari.qt.threading import thread_worker


class NellieProcessor(QWidget):
    def __init__(self, napari_viewer: 'napari.viewer.Viewer', nellie, parent=None):
        super().__init__(parent)
        self.nellie = nellie
        self.viewer = napari_viewer

        self.im_info_list = None
        self.current_im_info = None

        self.status_label = QLabel("Awaiting your input")
        self.status = None
        self.num_ellipses = 1

        self.status_timer = QTimer()
        self.status_timer.timeout.connect(self.update_status)

        self.open_dir_button = QPushButton(text="Open output directory")
        self.open_dir_button.clicked.connect(self.open_directory)
        self.open_dir_button.setEnabled(False)

        # Run im button
        self.run_button = QPushButton(text="Run Nellie")
        self.run_button.clicked.connect(self.run_nellie)
        self.run_button.setEnabled(True)
        self.run_button.setFixedWidth(200)
        self.run_button.setFixedHeight(100)
        self.run_button.setStyleSheet("border-radius: 10px;")
        self.run_button.setFont(QFont("Arial", 20))

        # Preprocess im button
        self.preprocess_button = QPushButton(text="Run preprocessing")
        self.preprocess_button.clicked.connect(self.run_preprocessing)
        self.preprocess_button.setEnabled(False)

        # Segment im button
        self.segment_button = QPushButton(text="Run segmentation")
        self.segment_button.clicked.connect(self.run_segmentation)
        self.segment_button.setEnabled(False)

        # Run mocap button
        self.mocap_button = QPushButton(text="Run mocap marking")
        self.mocap_button.clicked.connect(self.run_mocap)
        self.mocap_button.setEnabled(False)

        # Run tracking button
        self.track_button = QPushButton(text="Run tracking")
        self.track_button.clicked.connect(self.run_tracking)
        self.track_button.setEnabled(False)

        # Run reassign button
        self.reassign_button = QPushButton(text="Run voxel reassignment")
        self.reassign_button.clicked.connect(self.run_reassign)
        self.reassign_button.setEnabled(False)

        # Run feature extraction button
        self.feature_export_button = QPushButton(text="Run feature export")
        self.feature_export_button.clicked.connect(self.run_feature_export)
        self.feature_export_button.setEnabled(False)

        self.set_ui()

        self.initialized = False
        self.pipeline = False

    def set_ui(self):
        main_layout = QVBoxLayout()

        # Status group
        status_group = QGroupBox("Status")
        status_layout = QHBoxLayout()  # Changed to QHBoxLayout
        status_layout.addWidget(self.status_label, alignment=Qt.AlignLeft)
        status_layout.addWidget(self.open_dir_button, alignment=Qt.AlignCenter)
        status_group.setMaximumHeight(100)
        status_group.setLayout(status_layout)

        # Run full pipeline
        full_pipeline_group = QGroupBox("Run full pipeline")
        full_pipeline_layout = QVBoxLayout()
        full_pipeline_layout.addWidget(self.run_button, alignment=Qt.AlignCenter)
        full_pipeline_group.setLayout(full_pipeline_layout)

        # Run partial pipeline
        partial_pipeline_group = QGroupBox("Run individual steps")
        partial_pipeline_layout = QVBoxLayout()
        partial_pipeline_layout.addWidget(self.preprocess_button)
        partial_pipeline_layout.addWidget(self.segment_button)
        partial_pipeline_layout.addWidget(self.mocap_button)
        partial_pipeline_layout.addWidget(self.track_button)
        partial_pipeline_layout.addWidget(self.reassign_button)
        partial_pipeline_layout.addWidget(self.feature_export_button)
        partial_pipeline_group.setLayout(partial_pipeline_layout)

        main_layout.addWidget(status_group)
        main_layout.addWidget(full_pipeline_group)
        main_layout.addWidget(partial_pipeline_group)

        self.setLayout(main_layout)

    def post_init(self):
        if self.nellie.im_info_list is None:
            self.im_info_list = [self.nellie.im_info]
            self.current_im_info = self.nellie.im_info
        else:
            self.im_info_list = self.nellie.im_info_list
            self.current_im_info = self.nellie.im_info_list[0]
        self.check_file_existence()
        self.initialized = True
        self.open_dir_button.setEnabled(os.path.exists(self.current_im_info.file_info.output_dir))

    def check_file_existence(self):
        self.nellie.visualizer.check_file_existence()

        # set all other buttons to disabled first
        self.run_button.setEnabled(False)
        self.preprocess_button.setEnabled(False)
        self.segment_button.setEnabled(False)
        self.mocap_button.setEnabled(False)
        self.track_button.setEnabled(False)
        self.reassign_button.setEnabled(False)
        self.feature_export_button.setEnabled(False)

        analysis_path = self.current_im_info.pipeline_paths['features_organelles']
        if os.path.exists(analysis_path):
            self.nellie.setTabEnabled(self.nellie.analysis_tab, True)
        else:
            self.nellie.setTabEnabled(self.nellie.analysis_tab, False)

        self.open_dir_button.setEnabled(os.path.exists(self.current_im_info.file_info.output_dir))

        if os.path.exists(self.current_im_info.im_path):
            self.run_button.setEnabled(True)
            self.preprocess_button.setEnabled(True)
        else:
            return

        frangi_path = self.current_im_info.pipeline_paths['im_preprocessed']
        if os.path.exists(frangi_path):
            self.segment_button.setEnabled(True)
        else:
            self.segment_button.setEnabled(False)
            self.mocap_button.setEnabled(False)
            self.track_button.setEnabled(False)
            self.reassign_button.setEnabled(False)
            self.feature_export_button.setEnabled(False)
            return

        im_instance_label_path = self.current_im_info.pipeline_paths['im_instance_label']
        im_skel_relabelled_path = self.current_im_info.pipeline_paths['im_skel_relabelled']
        if os.path.exists(im_instance_label_path) and os.path.exists(im_skel_relabelled_path):
            self.mocap_button.setEnabled(True)
        else:
            self.mocap_button.setEnabled(False)
            self.track_button.setEnabled(False)
            self.reassign_button.setEnabled(False)
            self.feature_export_button.setEnabled(False)
            return

        im_marker_path = self.current_im_info.pipeline_paths['im_marker']
        if os.path.exists(im_marker_path):
            self.track_button.setEnabled(True)
        else:
            self.track_button.setEnabled(False)
            self.reassign_button.setEnabled(False)
            self.feature_export_button.setEnabled(False)
            return

        track_path = self.current_im_info.pipeline_paths['flow_vector_array']
        if os.path.exists(track_path):
            self.reassign_button.setEnabled(True)
            self.feature_export_button.setEnabled(True)
        else:
            self.reassign_button.setEnabled(False)
            self.feature_export_button.setEnabled(True)
            # if im_info's 'T' axis has more than 1 timepoint, disable the feature export button
            if self.current_im_info.shape[0] > 1:
                self.feature_export_button.setEnabled(False)
                return



    @thread_worker
    def _run_preprocessing(self):
        self.status = "preprocessing"
        for im_num, im_info in enumerate(self.im_info_list):
            show_info(f"Nellie is running: Preprocessing file {im_num + 1}/{len(self.im_info_list)}")
            self.current_im_info = im_info
            preprocessing = Filter(im_info=self.current_im_info,
                                   remove_edges=self.nellie.settings.remove_edges_checkbox.isChecked(),
                                   viewer=self.viewer)
            preprocessing.run()

    def run_preprocessing(self):
        worker = self._run_preprocessing()
        worker.started.connect(self.turn_off_buttons)
        if self.pipeline:
            worker.finished.connect(self.run_segmentation)
        worker.started.connect(self.set_status)
        worker.finished.connect(self.reset_status)
        worker.finished.connect(self.check_file_existence)
        worker.start()

    @thread_worker
    def _run_segmentation(self):
        self.status = "segmentation"
        for im_num, im_info in enumerate(self.im_info_list):
            show_info(f"Nellie is running: Segmentation file {im_num + 1}/{len(self.im_info_list)}")
            self.current_im_info = im_info
            segmenting = Label(im_info=self.current_im_info, viewer=self.viewer)
            segmenting.run()
            networking = Network(im_info=self.current_im_info, viewer=self.viewer)
            networking.run()

    def run_segmentation(self):
        worker = self._run_segmentation()
        worker.started.connect(self.turn_off_buttons)
        if self.pipeline:
            worker.finished.connect(self.run_mocap)
        worker.finished.connect(self.check_file_existence)
        worker.started.connect(self.set_status)
        worker.finished.connect(self.reset_status)
        worker.start()

    @thread_worker
    def _run_mocap(self):
        self.status = "mocap marking"
        for im_num, im_info in enumerate(self.im_info_list):
            show_info(f"Nellie is running: Mocap Marking file {im_num + 1}/{len(self.im_info_list)}")
            self.current_im_info = im_info
            mocap_marking = Markers(im_info=self.current_im_info, viewer=self.viewer)
            mocap_marking.run()

    def run_mocap(self):
        worker = self._run_mocap()
        worker.started.connect(self.turn_off_buttons)
        if self.pipeline:
            worker.finished.connect(self.run_tracking)
        worker.finished.connect(self.check_file_existence)
        worker.started.connect(self.set_status)
        worker.finished.connect(self.reset_status)
        worker.start()


    @thread_worker
    def _run_tracking(self):
        self.status = "tracking"
        for im_num, im_info in enumerate(self.im_info_list):
            show_info(f"Nellie is running: Tracking file {im_num + 1}/{len(self.im_info_list)}")
            self.current_im_info = im_info
            hu_tracking = HuMomentTracking(im_info=self.current_im_info, viewer=self.viewer)
            hu_tracking.run()

    def run_tracking(self):
        worker = self._run_tracking()
        worker.started.connect(self.turn_off_buttons)
        if self.pipeline:
            if self.nellie.settings.voxel_reassign.isChecked():
                worker.finished.connect(self.run_reassign)
            else:
                worker.finished.connect(self.run_feature_export)
        worker.finished.connect(self.check_file_existence)
        worker.started.connect(self.set_status)
        worker.finished.connect(self.reset_status)
        worker.start()

    @thread_worker
    def _run_reassign(self):
        self.status = "voxel reassignment"
        for im_num, im_info in enumerate(self.im_info_list):
            show_info(f"Nellie is running: Voxel Reassignment file {im_num + 1}/{len(self.im_info_list)}")
            self.current_im_info = im_info
            vox_reassign = VoxelReassigner(im_info=self.current_im_info, viewer=self.viewer)
            vox_reassign.run()

    def run_reassign(self):
        worker = self._run_reassign()
        worker.started.connect(self.turn_off_buttons)
        if self.pipeline:
            worker.finished.connect(self.run_feature_export)
        worker.finished.connect(self.check_file_existence)
        worker.started.connect(self.set_status)
        worker.finished.connect(self.reset_status)
        worker.start()


    @thread_worker
    def _run_feature_export(self):
        self.status = "feature export"
        for im_num, im_info in enumerate(self.im_info_list):
            show_info(f"Nellie is running: Feature export file {im_num + 1}/{len(self.im_info_list)}")
            self.current_im_info = im_info
            hierarchy = Hierarchy(im_info=self.current_im_info,
                                  skip_nodes=not bool(self.nellie.settings.analyze_node_level.isChecked()),
                                  viewer=self.viewer)
            hierarchy.run()
            if self.nellie.settings.remove_intermediates_checkbox.isChecked():
                try:
                    self.current_im_info.remove_intermediates()
                except Exception as e:
                    show_info(f"Error removing intermediates: {e}")
        if self.nellie.analyzer.initialized:
            self.nellie.analyzer.rewrite_dropdown()

    def run_feature_export(self):
        worker = self._run_feature_export()
        worker.started.connect(self.turn_off_buttons)
        worker.finished.connect(self.check_file_existence)
        worker.finished.connect(self.turn_off_pipeline)
        worker.started.connect(self.set_status)
        worker.finished.connect(self.reset_status)
        worker.start()

    def turn_off_pipeline(self):
        self.pipeline = False

    def run_nellie(self):
        self.pipeline = True
        self.run_preprocessing()

    def set_status(self):
        self.running = True
        self.status_timer.start(500)  # Update every 250 ms

    def update_status(self):
        if self.running:
            self.status_label.setText(f"Running {self.status}{'.' * (self.num_ellipses % 4)}")
            self.num_ellipses += 1
        else:
            self.status_timer.stop()

    def reset_status(self):
        self.running = False
        self.status_label.setText("Awaiting your input")
        self.num_ellipses = 1
        self.status_timer.stop()

    def turn_off_buttons(self):
        self.run_button.setEnabled(False)
        self.preprocess_button.setEnabled(False)
        self.segment_button.setEnabled(False)
        self.mocap_button.setEnabled(False)
        self.track_button.setEnabled(False)
        self.reassign_button.setEnabled(False)
        self.feature_export_button.setEnabled(False)

    def open_directory(self):
        directory = self.current_im_info.file_info.output_dir
        if os.path.exists(directory):
            if os.name == 'nt':  # For Windows
                os.startfile(directory)
            elif os.name == 'posix':  # For macOS and Linux
                subprocess.call(['open', directory])
        else:
            show_info("Output directory does not exist.")


if __name__ == "__main__":
    import napari
    viewer = napari.Viewer()
    napari.run()



import os
import napari
from PyQt5.QtWidgets import QSpinBox
from qtpy.QtWidgets import QWidget, QLabel, QPushButton, QLineEdit, QFileDialog, QVBoxLayout, QHBoxLayout, QGroupBox
from napari.utils.notifications import show_info
from tifffile import tifffile

from nellie.im_info.verifier import FileInfo, ImInfo


class NellieFileSelect(QWidget):
    def __init__(self, napari_viewer: 'napari.viewer.Viewer', nellie, parent=None):
        super().__init__(parent)
        self.nellie = nellie
        self.filepath = None
        self.file_info: FileInfo = None
        self.im_info = None

        self.batch_fileinfo_list = None

        self.viewer = napari_viewer
        self.viewer.title = 'Nellie Napari'
        # Add text showing what filepath is selected
        self.filepath_text = QLabel(text="No file selected.")
        self.filepath_text.setWordWrap(True)

        # Filepath selector button
        self.filepath_button = QPushButton(text="Select File")
        self.filepath_button.clicked.connect(self.select_filepath)
        self.filepath_button.setEnabled(True)

        # Folder selector button
        self.folder_button = QPushButton(text="Select Folder")
        self.folder_button.clicked.connect(self.select_folder)
        self.folder_button.setEnabled(True)

        self.selection_text = QLabel("Selected file:")
        self.selection_text.setWordWrap(True)

        self.reset_button = QPushButton(text="Reset")
        self.reset_button.clicked.connect(self.nellie.reset)
        self.reset_button.setEnabled(True)

        self.file_shape_text = QLabel(text="None")
        self.file_shape_text.setWordWrap(True)

        self.current_order_text = QLabel(text="None")
        self.current_order_text.setWordWrap(True)

        self.dim_order_button = QLineEdit(self)
        self.dim_order_button.setText("None")
        self.dim_order_button.setToolTip("Accepted axes: None")
        self.dim_order_button.setEnabled(False)
        self.dim_order_button.textChanged.connect(self.handle_dim_order_changed)

        self.dim_t_text = 'None'
        self.dim_z_text = 'None'
        self.dim_xy_text = 'None'

        self.label_t = QLabel("T resolution (s):")
        self.dim_t_button = QLineEdit(self)
        self.dim_t_button.setText("None")
        self.dim_t_button.setEnabled(False)
        self.dim_t_button.textChanged.connect(self.handle_t_changed)

        self.label_z = QLabel("Z resolution (um):")
        self.dim_z_button = QLineEdit(self)
        self.dim_z_button.setText("None")
        self.dim_z_button.setEnabled(False)
        self.dim_z_button.textChanged.connect(self.handle_z_changed)

        self.label_xy = QLabel("X & Y resolution (um):")
        self.dim_xy_button = QLineEdit(self)
        self.dim_xy_button.setText("None")
        self.dim_xy_button.setEnabled(False)
        self.dim_xy_button.textChanged.connect(self.handle_xy_changed)

        self.label_channel = QLabel("Channel:")
        self.channel_button = QSpinBox(self)
        self.channel_button.setRange(0, 0)
        self.channel_button.setValue(0)
        self.channel_button.setEnabled(False)
        self.channel_button.valueChanged.connect(self.change_channel)

        self.label_time = QLabel("Start frame:")
        self.label_time_2 = QLabel("End frame:")
        self.start_frame_button = QSpinBox(self)
        self.start_frame_button.setRange(0, 0)
        self.start_frame_button.setValue(0)
        self.start_frame_button.setEnabled(False)
        self.start_frame_button.valueChanged.connect(self.change_time)
        self.end_frame_button = QSpinBox(self)
        self.end_frame_button.setRange(0, 0)
        self.end_frame_button.setValue(0)
        self.end_frame_button.setEnabled(False)
        self.end_frame_button.valueChanged.connect(self.change_time)
        self.end_frame_init = False

        self.confirm_button = QPushButton(text="Confirm")
        self.confirm_button.clicked.connect(self.on_confirm)
        self.confirm_button.setEnabled(False)

        self.preview_button = QPushButton(text="Preview image")
        self.preview_button.clicked.connect(self.on_preview)
        self.preview_button.setEnabled(False)

        # self.delete_button = QPushButton(text="Delete image")

        self.process_button = QPushButton(text="Process image")
        self.process_button.clicked.connect(self.on_process)
        self.process_button.setEnabled(False)

        self.init_ui()

    def init_ui(self):
        main_layout = QVBoxLayout()

        # File Selection Group
        file_group = QGroupBox("File Selection")
        file_layout = QVBoxLayout()
        file_button_sublayout = QHBoxLayout()
        file_button_sublayout.addWidget(self.filepath_button)
        file_button_sublayout.addWidget(self.folder_button)
        file_layout.addLayout(file_button_sublayout)
        file_sub_layout = QHBoxLayout()
        file_sub_layout.addWidget(self.selection_text)
        file_sub_layout.addWidget(self.filepath_text)
        file_layout.addLayout(file_sub_layout)
        file_layout.addWidget(self.reset_button)
        file_group.setLayout(file_layout)

        # Axes Info Group
        axes_group = QGroupBox("Axes Information")
        axes_layout = QVBoxLayout()
        sub_layout = QHBoxLayout()
        sub_layout.addWidget(QLabel("Dimension order:"))
        sub_layout.addWidget(self.dim_order_button)
        sub_layout.addWidget(QLabel("Only T, C, Z, Y, and X allowed."))
        axes_layout.addLayout(sub_layout)
        for label, button in [
            (QLabel("File shape:"), self.file_shape_text),
            (QLabel("Current order:"), self.current_order_text)
        ]:
            sub_layout = QHBoxLayout()
            sub_layout.addWidget(label)
            sub_layout.addWidget(button)
            axes_layout.addLayout(sub_layout)
        axes_group.setLayout(axes_layout)

        # Dimensions Group
        dim_group = QGroupBox("Dimension Resolutions")
        dim_layout = QVBoxLayout()
        for label, button in [
            (self.label_t, self.dim_t_button),
            (self.label_z, self.dim_z_button),
            (self.label_xy, self.dim_xy_button)
        ]:
            sub_layout = QHBoxLayout()
            sub_layout.addWidget(label)
            sub_layout.addWidget(button)
            dim_layout.addLayout(sub_layout)
        dim_group.setLayout(dim_layout)

        # Slice Settings Group
        slice_group = QGroupBox("Slice Settings")
        slice_layout = QVBoxLayout()
        for label, button in [
            (self.label_time, self.start_frame_button),
            (self.label_time_2, self.end_frame_button),
            (self.label_channel, self.channel_button)
        ]:
            sub_layout = QHBoxLayout()
            sub_layout.addWidget(label)
            sub_layout.addWidget(button)
            slice_layout.addLayout(sub_layout)
        slice_group.setLayout(slice_layout)

        # Action Buttons Group
        action_group = QGroupBox("Actions")
        action_layout = QHBoxLayout()
        action_layout.addWidget(self.confirm_button)
        action_layout.addWidget(self.preview_button)
        action_layout.addWidget(self.process_button)
        action_group.setLayout(action_layout)

        # Add all groups to main layout
        main_layout.addWidget(file_group)
        main_layout.addWidget(axes_group)
        main_layout.addWidget(dim_group)
        main_layout.addWidget(slice_group)
        main_layout.addWidget(action_group)

        self.setLayout(main_layout)

    def select_filepath(self):
        self.batch_fileinfo_list = None
        filepath, _ = QFileDialog.getOpenFileName(self, "Select file")
        self.validate_path(filepath)
        if self.filepath is None:
            return
        self.selection_text.setText("Selected file:")

        self.file_info = FileInfo(self.filepath)
        self.initialize_single_file()
        filename = os.path.basename(self.filepath)
        self.filepath_text.setText(f"{filename}")

    def select_folder(self):
        folderpath = QFileDialog.getExistingDirectory(self, "Select folder")
        self.validate_path(folderpath)
        if self.filepath is None:
            return
        self.selection_text.setText("Selected folder:")

        self.initialize_folder()
        self.filepath_text.setText(f"{folderpath}")


    def validate_path(self, filepath):
        if not filepath:
            show_info("Invalid selection.")
            return None
        self.filepath = filepath

    def initialize_single_file(self):
        self.file_info.find_metadata()
        self.file_info.load_metadata()
        self.file_shape_text.setText(f"{self.file_info.shape}")

        self.dim_order_button.setText(f"{self.file_info.axes}")
        self.dim_order_button.setEnabled(True)
        self.on_change()

    def initialize_folder(self):
        # get all .tif, .tiff, and .nd2 files in the folder
        files = [f for f in os.listdir(self.filepath) if f.endswith('.tif') or f.endswith('.tiff') or f.endswith('.nd2')]
        # for each file, create a FileInfo object
        self.batch_fileinfo_list = [FileInfo(os.path.join(self.filepath, f)) for f in files]
        for file_info in self.batch_fileinfo_list:
            file_info.find_metadata()
            file_info.load_metadata()
        self.file_info = self.batch_fileinfo_list[0]
        self.initialize_single_file()
        # This assumes all files in the folder have the same metadata (dim order, resolutions, temporal range, channels)

    def on_change(self):
        self.confirm_button.setEnabled(False)
        self.check_available_dims()
        if len(self.file_info.shape) == 2:
            self.dim_order_button.setToolTip("Accepted axes: 'Y', 'X' (e.g. 'YX')")
        elif len(self.file_info.shape) == 3:
            self.dim_order_button.setToolTip("Accepted axes: ['T' or 'C' or 'Z'], 'Y', 'X' (e.g. 'ZYX')")
        elif len(self.file_info.shape) == 4:
            self.dim_order_button.setToolTip("Accepted axes: ['T' or 'C' or 'Z']x2, 'Y', 'X' (e.g. 'TZYX')")
        elif len(self.file_info.shape) == 5:
            self.dim_order_button.setToolTip("Accepted axes: ['T' or 'C' or 'Z']x3, 'Y', 'X' (e.g. 'TZCYX')")
        elif len(self.file_info.shape) > 5:
            self.dim_order_button.setStyleSheet("background-color: red")
            show_info(f"Error: Too many dimensions found ({self.file_info.shape}).")

        if self.file_info.good_axes:
            current_order_text = f"({', '.join(self.file_info.axes)})"
            self.current_order_text.setText(current_order_text)
            self.dim_order_button.setStyleSheet("background-color: green")
        else:
            self.current_order_text.setText("Invalid")
            self.dim_order_button.setStyleSheet("background-color: red")

        if self.file_info.good_dims and self.file_info.good_axes:
            self.confirm_button.setEnabled(True)

        # self.delete_button.setEnabled(False)
        self.preview_button.setEnabled(False)
        self.process_button.setEnabled(False)
        # check file_info output path. if it exists, enable the delete and preview button
        if os.path.exists(self.file_info.ome_output_path) and self.file_info.good_dims and self.file_info.good_axes:
            # self.delete_button.setEnabled(True)
            self.preview_button.setEnabled(True)
            self.process_button.setEnabled(True)

    def check_available_dims(self):
        def check_dim(dim, dim_button, dim_text):
            dim_button.setStyleSheet("background-color: green")
            if dim in self.file_info.axes:
                dim_button.setEnabled(True)
                if dim_text is None or dim_text == 'None':
                    dim_button.setText(str(self.file_info.dim_res[dim]))
                else:
                    dim_button.setText(dim_text)
                    if self.file_info.dim_res[dim] is None:
                        dim_button.setStyleSheet("background-color: red")
                if dim_button.text() == 'None' or dim_button.text() is None:
                    dim_button.setStyleSheet("background-color: red")
            else:
                dim_button.setEnabled(False)
                if dim in self.file_info.dim_res:
                    dim_button.setText(str(self.file_info.dim_res[dim]))
                else:
                    dim_button.setText("None")

        check_dim('T', self.dim_t_button, self.dim_t_text)
        check_dim('Z', self.dim_z_button, self.dim_z_text)
        check_dim('X', self.dim_xy_button, self.dim_xy_text)
        check_dim('Y', self.dim_xy_button, self.dim_xy_text)

        self.channel_button.setEnabled(False)
        if 'C' in self.file_info.axes:
            self.channel_button.setEnabled(True)
            self.channel_button.setRange(0, self.file_info.shape[self.file_info.axes.index('C')]-1)

        self.start_frame_button.setEnabled(False)
        self.end_frame_button.setEnabled(False)
        if 'T' in self.file_info.axes:
            self.start_frame_button.setEnabled(True)
            self.end_frame_button.setEnabled(True)
            current_start_frame = self.start_frame_button.value()
            current_end_frame = self.end_frame_button.value()
            max_t = self.file_info.shape[self.file_info.axes.index('T')] - 1
            self.start_frame_button.setRange(0, current_end_frame)
            self.end_frame_button.setRange(current_start_frame, max_t)
            if not self.end_frame_init:
                self.start_frame_button.setValue(0)
                self.end_frame_button.setValue(max_t)
                self.end_frame_init = True

    def handle_dim_order_changed(self, text):
        if self.batch_fileinfo_list is None:
            self.file_info.change_axes(text)
        else:
            for file_info in self.batch_fileinfo_list:
                file_info.change_axes(text)

        self.end_frame_init = False
        self.on_change()

    def handle_t_changed(self, text):
        self.dim_t_text = text
        try:
            value = float(self.dim_t_text)
            if self.batch_fileinfo_list is None:
                self.file_info.change_dim_res('T', value)
            else:
                for file_info in self.batch_fileinfo_list:
                    file_info.change_dim_res('T', value)
        except ValueError:
            if self.batch_fileinfo_list is None:
                self.file_info.change_dim_res('T', None)
            else:
                for file_info in self.batch_fileinfo_list:
                    file_info.change_dim_res('T', None)
        self.on_change()

    def handle_z_changed(self, text):
        self.dim_z_text = text
        try:
            value = float(self.dim_z_text)
            if self.batch_fileinfo_list is None:
                self.file_info.change_dim_res('Z', value)
            else:
                for file_info in self.batch_fileinfo_list:
                    file_info.change_dim_res('Z', value)
        except ValueError:
            if self.batch_fileinfo_list is None:
                self.file_info.change_dim_res('Z', None)
            else:
                for file_info in self.batch_fileinfo_list:
                    file_info.change_dim_res('Z', None)
        self.on_change()

    def handle_xy_changed(self, text):
        self.dim_xy_text = text
        try:
            value = float(self.dim_xy_text)
            if self.batch_fileinfo_list is None:
                self.file_info.change_dim_res('X', value)
                self.file_info.change_dim_res('Y', value)
            else:
                for file_info in self.batch_fileinfo_list:
                    file_info.change_dim_res('X', value)
                    file_info.change_dim_res('Y', value)
        except ValueError:
            if self.batch_fileinfo_list is None:
                self.file_info.change_dim_res('X', None)
                self.file_info.change_dim_res('Y', None)
            else:
                for file_info in self.batch_fileinfo_list:
                    file_info.change_dim_res('X', None)
                    file_info.change_dim_res('Y', None)
        self.on_change()

    def change_channel(self):
        if self.batch_fileinfo_list is None:
            self.file_info.change_selected_channel(self.channel_button.value())
        else:
            for file_info in self.batch_fileinfo_list:
                file_info.change_selected_channel(self.channel_button.value())
        self.on_change()

    def change_time(self):
        if self.batch_fileinfo_list is None:
            self.file_info.select_temporal_range(self.start_frame_button.value(), self.end_frame_button.value())
        else:
            for file_info in self.batch_fileinfo_list:
                file_info.select_temporal_range(self.start_frame_button.value(), self.end_frame_button.value())
        self.on_change()

    def on_confirm(self):
        show_info("Saving OME TIFF file.")
        if self.batch_fileinfo_list is None:
            self.im_info = ImInfo(self.file_info)
        else:
            self.im_info = [ImInfo(file_info) for file_info in self.batch_fileinfo_list]
        self.on_change()

    def on_process(self):
        # switch to process tab
        if self.batch_fileinfo_list is None:
            self.im_info = ImInfo(self.file_info)
        else:
            self.im_info = [ImInfo(file_info) for file_info in self.batch_fileinfo_list]
        self.on_change()
        self.nellie.go_process()

    def on_preview(self):
        im_memmap = tifffile.memmap(self.file_info.ome_output_path)
        # num_t = min(2, self.im_info.shape[self.im_info.axes.index('T')])
        if 'Z' in self.file_info.axes:
            scale = (self.file_info.dim_res['Z'], self.file_info.dim_res['Y'], self.file_info.dim_res['X'])
            self.viewer.dims.ndisplay = 3
        else:
            scale = (self.file_info.dim_res['Y'], self.file_info.dim_res['X'])
            self.viewer.dims.ndisplay = 2
        self.viewer.add_image(im_memmap, name=self.file_info.filename_no_ext, scale=scale, blending='additive',
                              interpolation3d='nearest', interpolation2d='nearest')
        self.viewer.scale_bar.visible = True
        self.viewer.scale_bar.unit = 'um'


if __name__ == "__main__":
    import napari
    viewer = napari.Viewer()
    napari.run()



from qtpy.QtWidgets import QLabel, QVBoxLayout, QWidget, QPushButton, QMessageBox
from qtpy.QtGui import QPixmap, QFont
from qtpy.QtCore import Qt
import napari
import os
import datetime
import numpy as np
from napari.utils.notifications import show_info
import matplotlib.image


class Home(QWidget):
    def __init__(self, napari_viewer: 'napari.viewer.Viewer', nellie, parent=None):
        super().__init__(parent)
        self.nellie = nellie
        self.viewer = napari_viewer

        self.layout = QVBoxLayout()
        self.setLayout(self.layout)

        # Logo
        logo_path = os.path.join(os.path.dirname(__file__), 'logo.png')
        logo_label = QLabel(self)
        pixmap = QPixmap(logo_path)
        logo_label.setPixmap(
            pixmap.scaled(300, 300, Qt.KeepAspectRatio))#, Qt.SmoothTransformation))  # Adjust size as needed
        logo_label.setAlignment(Qt.AlignCenter)
        self.layout.addWidget(logo_label)

        # Title
        title = QLabel("Nellie")
        title.setFont(QFont("Arial", 24, QFont.Bold))
        title.setAlignment(Qt.AlignCenter)
        self.layout.addWidget(title)

        # Subtitle
        subtitle = QLabel("Automated organelle segmentation, tracking, and hierarchical feature extraction in 2D/3D live-cell microscopy.")
        subtitle.setFont(QFont("Arial", 16))
        subtitle.setAlignment(Qt.AlignCenter)
        subtitle.setWordWrap(True)
        self.layout.addWidget(subtitle)

        # Add a large "Start" button
        self.start_button = QPushButton("Start")
        self.start_button.setFont(QFont("Arial", 20))
        self.start_button.setFixedWidth(200)
        self.start_button.setFixedHeight(100)
        # rounded-edges
        self.start_button.setStyleSheet("border-radius: 10px;")
        # opens the file select tab
        self.start_button.clicked.connect(lambda: self.nellie.setCurrentIndex(self.nellie.file_select_tab))
        self.layout.addWidget(self.start_button, alignment=Qt.AlignCenter)

        github_link = QLabel("<a href='https://arxiv.org/abs/2403.13214'>Cite our paper!</a>")
        github_link.setOpenExternalLinks(True)
        github_link.setAlignment(Qt.AlignCenter)
        self.layout.addWidget(github_link)

        # Link to GitHub
        github_link = QLabel("<a href='https://github.com/aelefebv/nellie'>Visit Nellie's GitHub Page!</a>")
        github_link.setOpenExternalLinks(True)
        github_link.setAlignment(Qt.AlignCenter)
        self.layout.addWidget(github_link)

        # screenshot button
        self.screenshot_button = QPushButton(text="Easy screenshot:\n[Ctrl-Shift-E]")
        self.screenshot_button.setStyleSheet("border-radius: 5px;")
        self.screenshot_button.clicked.connect(self.screenshot)
        self.screenshot_button.setEnabled(True)
        self.viewer.bind_key('Ctrl-Shift-E', self.screenshot, overwrite=True)

        self.layout.addWidget(self.screenshot_button, alignment=Qt.AlignCenter)

    def screenshot(self, event=None):
        if self.nellie.im_info is None:
            show_info("No file selected, cannot take screenshot")
            return

        # easy no prompt screenshot
        dt = datetime.datetime.now()  # year, month, day, hour, minute, second, millisecond up to 3 digits
        dt = dt.strftime("%Y%m%d_%H%M%S%f")[:-3]

        screenshot_folder = self.nellie.im_info.screenshot_dir
        if not os.path.exists(screenshot_folder):
            os.makedirs(screenshot_folder)

        im_name = f'{dt}-{self.nellie.im_info.file_info.filename_no_ext}.png'
        file_path = os.path.join(screenshot_folder, im_name)

        # Take screenshot
        screenshot = self.viewer.screenshot(canvas_only=True)

        # Save the screenshot
        try:
            # save as png to file_path using imsave
            screenshot = np.ascontiguousarray(screenshot)
            matplotlib.image.imsave(file_path, screenshot, format="png")
            show_info(f"Screenshot saved to {file_path}")
        except Exception as e:
            QMessageBox.warning(None, "Error", f"Failed to save screenshot: {str(e)}")
            raise e


if __name__ == "__main__":
    import napari
    viewer = napari.Viewer()
    napari.run()



from qtpy.QtWidgets import QWidget, QCheckBox, QSpinBox, QLabel, QVBoxLayout, QGroupBox, QHBoxLayout
import napari


class Settings(QWidget):
    def __init__(self, napari_viewer: 'napari.viewer.Viewer', nellie, parent=None):
        super().__init__(parent)
        self.nellie = nellie
        self.viewer = napari_viewer

        # Checkbox for 'Remove edges'
        self.remove_edges_checkbox = QCheckBox("Remove image edges")
        self.remove_edges_checkbox.setChecked(False)
        self.remove_edges_checkbox.setEnabled(True)
        self.remove_edges_checkbox.setToolTip(
            "Originally for Snouty deskewed images. If you see weird image edge artifacts, enable this.")

        self.remove_intermediates_checkbox = QCheckBox("Remove intermediate files")
        self.remove_intermediates_checkbox.setChecked(False)
        self.remove_intermediates_checkbox.setEnabled(True)
        self.remove_intermediates_checkbox.setToolTip(
            "Remove intermediate files after processing. This means only csv files will be saved.")

        self.voxel_reassign = QCheckBox("Auto-run voxel reassignment")
        self.voxel_reassign.setChecked(False)
        self.voxel_reassign.setEnabled(True)

        # Analyze node level
        self.analyze_node_level = QCheckBox("Analyze node level (slow)")
        self.analyze_node_level.setChecked(False)
        self.analyze_node_level.setEnabled(True)

        # Track all frames
        self.track_all_frames = QCheckBox("Track all frames' voxels")
        self.track_all_frames.setChecked(True)
        self.track_all_frames.setEnabled(True)

        # Label above the spinner box
        self.skip_vox_label = QLabel("Track every N voxel. N=")

        self.skip_vox = QSpinBox()
        self.skip_vox.setRange(1, 10000)
        self.skip_vox.setValue(5)
        self.skip_vox.setEnabled(False)

        self.set_ui()

        self.initialized = False

    def post_init(self):
        self.initialized = True

    def set_ui(self):
        main_layout = QVBoxLayout()

        # Processor settings
        processor_group = QGroupBox("Processor settings")
        processor_layout = QVBoxLayout()
        subprocessor_layout1 = QHBoxLayout()
        subprocessor_layout1.addWidget(self.remove_intermediates_checkbox)
        subprocessor_layout1.addWidget(self.remove_edges_checkbox)
        subprocessor_layout2 = QHBoxLayout()
        subprocessor_layout2.addWidget(self.analyze_node_level)
        subprocessor_layout2.addWidget(self.voxel_reassign)
        processor_layout.addLayout(subprocessor_layout1)
        processor_layout.addLayout(subprocessor_layout2)
        processor_group.setLayout(processor_layout)

        # Tracking settings
        tracking_group = QGroupBox("Tracking settings")
        tracking_layout = QVBoxLayout()
        tracking_layout.addWidget(self.track_all_frames)
        skip_vox_layout = QHBoxLayout()
        skip_vox_layout.addWidget(self.skip_vox_label)
        skip_vox_layout.addWidget(self.skip_vox)
        tracking_layout.addLayout(skip_vox_layout)
        tracking_group.setLayout(tracking_layout)

        main_layout.addWidget(processor_group)
        main_layout.addWidget(tracking_group)
        self.setLayout(main_layout)


if __name__ == "__main__":
    import napari
    viewer = napari.Viewer()
    napari.run()
